unit jdm84io;           { PIC-Programmer for Turbo Pascal 6.0 - Ver 3.2 }
interface               { Copyright (c) 1997-1998, Jens Dyekjr Madsen  }

const
  base: integer = 0;                            { Com-port base adress  }
  din: byte = 0;                                { Returns PIC xxx data  }
  err: byte = 1;                                { Returns error code    }
  errtxt: array[0..5] of string[22] =           { Define errortxt       }
    ('No Error Found',                          { Error code 0 text     }
     'Comport not found',                       { Error code 1 text     }
     'Modem found',                             { Error code 2 text     }
     'Ilegal parameter value',                  { Error code 3 text     }
     'Programmer not found',                    { Error code 4 text     }
     'Cable too long');                         { Error code 5 text     }

procedure setcom(n: byte);                      { Select com-port / go  }
{
  Initialize the comport and perform hadware check. Return result in err.
}

procedure releasepg;                            { End comport use / go  }
{
  Release programmer. 'Automatic go' is activated after relpgm.
}

procedure begincmd;                             { Begin command         }
{ Indicates begin of command. Should be called before each PIC-command. }

procedure endcmd;                               { End command           }
{ Indicates end of command. Should be called after each PIC-command.    }


{ PIC routines }

procedure mclr;                                 { Reset PIC xxx         }

procedure dout(val: byte);                      { In/Out PIC16C84 data  }
{
  Send a bit of the value val to PIC16C84. Recieve data to din.
  Do only call Dout, SDA and SDL between begcmd and endcmd.
}


{ EEPROM (I2C) routines }

procedure sda(val: byte);                       { Set SDA = val         }
procedure scl(val: byte);                       { Set SCL = val         }
function rda: byte;                             { Return SDA            }
{
  SDA sets the value of data. SCL sets the value of clock. 
  Do only call Dout, SDA and SDL between begcmd and endcmd.
  RDA recieves data. Do not use RDA between begcmd and endcmd
  when waiting for acknoledge.
}


{ Delay routines }

procedure waitdelay(t: integer);                { Delay / ms            }
{ Delay of t ms (minimum, may be longer). About 87s if t=0. Max 5000ms.}


implementation

uses delays;                                    { Uses delay routines   }

var
  lastout: byte;                                { Saves last output     }

procedure setdlab(d: word);                     { Set Baud Rate Divisor }
var
  lcr: byte;                                    { Saves Line Control Reg}
begin
  lcr:=port[base+3];                            { Store Line Control Reg}
  port[base+3]:=$80 or lcr;                     { Select DLAB           }
  port[base]:=lo(d); port[base+1]:=hi(d);       { Set Baud rate         }
  port[base+3]:=lcr;                            { Restore LCR           }
end;

procedure getdlab(var d: word);                 { Get Baud Rate Divisor }
var
  lcr: byte;                                    { Saves Line Control Reg}
begin
  lcr:=port[base+3];                            { Store Line Control Reg}
  port[base+3]:=$80 or lcr;                     { Select DLAB           }
  d:=port[base]; d:=port[base+1] shl 8 + d;     { Get Baud rate         }
  port[base+3]:=lcr;                            { Restore LCR           }
end;

procedure waitdelay(t: integer);                { Delay / ms            }
begin
  setdlab((longint(t)*1152)div 100+1);          { Calculate baud rate   }
  port[base]:=0;                                { Send data             }
  repeat until port[base+5]>=96;		{ Wait to data send	}
end;

procedure check;                                { Check programmer on   }
var
  i :integer;                                   { Counter               }
  x :byte;                                      { Store data from port  }
begin
  err:=1;                                       { Assume error 1        }
  if base=0 then exit;                          { Exit if comport error }
                                                { Setup USART port      }
  port[base+2]:=6;                              { Disable fifo          }
  port[base+3]:=3;                              { 8 bit, no parity, IER }
  port[base+1]:=0;                              { No interrupts         }

  err:=2;                                       { Assume modem error (2)}
  port[base+4]:=0;                              { RTS low, DTR low      }
  waitdelay(1);                                 { Wait 1ms              }
  if port[base+6] and $10 <> 0 then exit;       { Exit if modem         }

  err:=0;                                       { Assume no error       }
  for i:=0 to 20 do                             { Clock data out, EEPROM}
  begin
    port[base+4]:=succ((i and 1)shl 1);         { DTR high, Toggle RTS  }
    waitdelay(0);                               { Wait 87s             }
  end;
  for i:=1 to 40 do                             { Check if programmer ok}
  begin
    shortdelay_on;                              { Use shortdelay, CLI   }
    port[base+4]:=i and 1;                      { RTS low, Toggle DTR   }
    shortdelay;                                 { Short delay           }
    x:=port[base+6];                            { Read data             }
    shortdelay_off;                             { End of shortdelay, STI}
    waitdelay(1);                               { Wait 1ms              }
    if ((x shr 4) xor i)and 1<>0 then err:=5;   { Check if error        }
    if ((port[base+6]shr 4)xor i)and 1<>0 then err:=4;   { Check err 4  }
    if err=5 then exit;                         { Exit if cable too long}
  end;                                          { RTS low, DTR low      }

  port[base+4]:=lastout;                        { Restore last output   }
end;


var
  mstate: array[1..4] of byte;                  { Saves USART state     }
  dlab: word;                                   { Stores Baud Rate Div. }

procedure setcom(n: byte);                      { Setup com-port        }
var                                             { And store com-state   }
  i: integer;                                   { Counter to store state}
begin
  err:=3;                                       { Error 3 if no comport }
  if (n=0) or (n>4) then exit;                  { Check comport value   }
  base:=memw[$40:2*pred(n)];                    { Setup RS232 port      }
  err:=1;                                       { Error 1 if not found  }
  if base=0 then exit;                          { Exit if not found     }

  for i:=2 to 4 do mstate[i]:=port[base+i];     { Store USART state     }
  port[base+3]:=3;                              { Select IER            }
  mstate[1]:=port[base+1];                      { Store IER             }
  getdlab(dlab);                                { Get Baud Rate Divisor }

  lastout:=0;                                   { Setup default output  }
  check;                                        { Check hardware        }
  if err<>0 then                                { If error then         }
  begin                                         { Restore USART state   }
    setdlab(dlab);                              { Set Baud Rate Divisor }
    for i:=1 to 4 do port[base+i]:=mstate[i];   { Restore USART state   }
  end;
end;

procedure releasepg;                            { End communication use }
begin
  check;                                        { Perform hardware check}
end;


{ Begin and end command }

procedure begincmd;                             { Begin command         }
begin                                           { No interrupts at cmd  }
  shortdelay_on;                                { CLI, start timer 2    }
end;

procedure endcmd;                               { End command           }
begin                                           { Interrupt ok after cmd}
  shortdelay_off;                               { Close timer 2, STI    }
end;


{ PIC xxxx routines }

procedure mclr;                                 { Reset PIC xxx         }
var
  brkstate: byte;                               { Store break state     }
  i: byte;                                      { Used to count         }
begin
  brkstate:=port[base+3];                       { Save break state      }
  check;                                        { Perform hardware check}
  if err=0 then                                 { Check to be sure that }
  begin                                         { No modem is connected }
    if brkstate and $40 = 0 then                { If TxD is low, then   }
    begin                                       { Power up denied       }
      port[base+3]:=$43;                        { Enable Break, MCLR on }
      waitdelay(350);                           { Power on Delay, 350ms }
    end;
    setdlab(1);                                 { Setup 115Kbaud,n,8,1  }
    port[base+3]:=3;                            { Start reset, MCLR low }
    for i:=1 to 3 do                            { pulses                }
    begin                                       { to not start auto go  }
      port[base]:=0;                                    { Output zero   }
      repeat until boolean(port[base+5] and $20);       { Wait on empty }
    end;
    port[base+3]:=$43;                          { Enable break, no reset}
    waitdelay(0);                               { Delay 87s            }
  end;
end;

procedure dout(val: byte);                      { Output PIC data       }
var
  i: integer;                                   { Used to count         }
begin
  port[base+4]:=val or 2;                       { Output value, Clk high}
  shortdelay;                                   { Wait on data          }
  din:=(port[base+6] shr 4) and 1;              { Return data           }
  port[base+4]:=val and din;                    { Output value, Clk low }
  shortdelay;                                   { Recharge              }
  if din<>val then for i:=1 to 4 do shortdelay; { Extra rechage delay   }
end;


{ EEPROM (I2C) routines }

function rda: byte;                             { Read data             }
begin
  rda:=(port[base+6] shr 4) and 1;              { Setup rda value       }
end;

procedure sda(val: byte);                       { Set SDA = val         }
begin
  lastout:=(lastout and $fe) or val;            { Update lastout        }
  port[base+4]:=lastout;                        { Output lastout        }
  shortdelay;                                   { Short delay           }
end;

procedure scl(val: byte);                       { Set SCL = val         }
begin
  lastout:=(lastout and $fd) or 2*val;          { Update lastout        }
  shortdelay;                                   { Short delay           }
end;

end.
